// ------------------------------------------------------------------------------------
// <copyright file="IsbnBL.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.Negocio.Clases.BL
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Biblioteca.Datos.Clases.DAL.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.IC.DTO.Repositorio;
	using Biblioteca.IC.Recursos;
	using TransaccionalNet.EF.Clases;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using Utilitarios.TransversalesNet.Clases.Comunes.Enumeraciones;

	/// <summary>
	/// Clase con las acciones de negocio de la entidad Isbn
	/// </summary>
	public class IsbnBL : AccesoComunBL, IIsbnNegocioAccion
	{

		#region ATRIBUTOS
		/// <summary>
		/// Objeto para repositorio Isbn
		/// <summary>
		/// Objeto para acciones Isbn
		/// </summary>
		private Lazy<IIsbnRepositorioAccion> isbnRepositorioAccion;

		/// <summary>
		/// Objeto para entidad respuesta
		/// </summary>
		Respuesta<IIsbnDTO> respuesta;

		#endregion ATRIBUTOS
		#region CONSTRUCTORES
		/// <summary>
		/// Inicializa una nueva instancia de la clase IsbnBL
		/// </summary>
		/// <param name="argIsbnRepositorioAccion">Acciones entidad Isbn</param>
		public IsbnBL(Lazy<IIsbnRepositorioAccion> argIsbnRepositorioAccion = null)
		{
			this.respuesta = new Respuesta<IIsbnDTO>();
			this.isbnRepositorioAccion = argIsbnRepositorioAccion ?? new Lazy<IIsbnRepositorioAccion>(() => new IsbnDAL());
		}

		#endregion CONSTRUCTORES
		#region METODOS PUBLICOS
		/// <summary>
		/// Metodo consultar lista isbn
		/// </summary>
		/// <returns>Respuesta tipo Isbn </returns>
		public async Task<Respuesta<IIsbnDTO>> ConsultarListaIsbnAsync()
		{

			return await this.EjecutarTransaccionBDAsync<Respuesta<IIsbnDTO>, IsbnBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.isbnRepositorioAccion.Value.ConsultarListaIsbnAsync();
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});

		}
		/// <summary>
		/// Metodo consultar por llave isbn
		/// </summary>
		/// <param name="isbn">Entidad a consultar</param>
		/// <returns>Respuesta tipo Isbn </returns>
		public async Task<Respuesta<IIsbnDTO>> ConsultarIsbnLlaveAsync(IIsbnDTO isbn)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IIsbnDTO>, IsbnBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.isbnRepositorioAccion.Value.ConsultarIsbnLlaveAsync(isbn);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo editar isbn
		/// </summary>
		/// <param name="isbn">Entidad a editar</param>
		/// <returns>Respuesta tipo Isbn </returns>
		public async Task<Respuesta<IIsbnDTO>> EditarIsbnAsync(IIsbnDTO isbn)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IIsbnDTO>, IsbnBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.isbnRepositorioAccion.Value.EditarIsbnAsync(isbn);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}


		/// <summary>
		/// Metodo eliminar isbn
		/// </summary>
		/// <param name="isbn">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Isbn </returns>
		public async Task<Respuesta<IIsbnDTO>> EliminarIsbnAsync(IIsbnDTO isbn)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IIsbnDTO>, IsbnBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.isbnRepositorioAccion.Value.EliminarIsbnAsync(isbn);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeEntidadEliminadaConExito);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo guardar isbn
		/// </summary>
		/// <param name="isbn">Entidad a guardar</param>
		/// <returns>Respuesta tipo Isbn </returns>
		public async Task<Respuesta<IIsbnDTO>> GuardarIsbnAsync(IIsbnDTO isbn)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IIsbnDTO>, IsbnBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.isbnRepositorioAccion.Value.GuardarIsbnAsync(isbn);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo Guardar lista isbn
		/// </summary>
		/// <param name="listaIsbn">Lista de entidades a guardar</param>
		/// <returns>Respuesta tipo Isbn </returns>
		public async Task<Respuesta<IIsbnDTO>> GuardarListaIsbnAsync(List<IIsbnDTO> listaIsbn)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IIsbnDTO>, IsbnBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.isbnRepositorioAccion.Value.GuardarListaIsbnAsync(listaIsbn);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}
		#endregion METODOS PUBLICOS
		#region METODOS PRIVADOS
		#endregion METODOS PRIVADOS
	}
}

