// ------------------------------------------------------------------------------------
// <copyright file="IsbnController.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.API.Controllers
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Utilitarios.APINet.Clases.Ejecucion;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using UtilitariosNet.Clases.Mapeador.Extensiones;
	using Biblioteca.API.Models.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.Negocio.Clases.BL;
	using Microsoft.AspNetCore.Mvc;
	
	/// <summary>
	/// Clase con las capaciades Rest de la entidad Isbn
	/// </summary>
	[Route("api/Isbn")]
	public class IsbnController : AccesoComunAPI
	{


		/// <summary>
		/// Objeto para negocio Isbn
		/// </summary>
		private Lazy<IIsbnNegocioAccion> negocioIsbn;

		
		/// <summary>
		/// Inicializa una nueva instancia de la clase IsbnController
		/// </summary>
		public IsbnController()
		{
			this.negocioIsbn = new Lazy<IIsbnNegocioAccion>(
										() => new IsbnBL());
		}
		
		/// <summary>
		/// Metodo consultar lista isbn
		/// </summary>
		/// <returns>Respuesta tipo Isbn </returns>
		[HttpGet]
		[Route("ConsultarListaIsbn")]
		public async Task<Respuesta<Isbn>> ConsultarListaIsbn()
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Isbn>>, IsbnController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Isbn>>(await negocioIsbn.Value.ConsultarListaIsbnAsync());
			});
		}
		
		/// <summary>
		/// Metodo consultar por llave isbn
		/// </summary>
		/// <param name="isbn">Entidad a consultar</param>
		/// <returns>Respuesta tipo Isbn </returns>
		[HttpPost]
		[Route("ConsultarIsbnLlave")]
		public async Task<Respuesta<Isbn>> ConsultarIsbnLlave(Isbn isbn)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Isbn>>, IsbnController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Isbn>>(await negocioIsbn.Value.ConsultarIsbnLlaveAsync(isbn));
			});
		}
		
		/// <summary>
		/// Metodo guardar isbn
		/// </summary>
		/// <param name="isbn">Entidad a guardar</param>
		/// <returns>Respuesta tipo Isbn </returns>
		[HttpPost]
		[Route("GuardarIsbn")]
		public async Task<Respuesta<Isbn>> GuardarIsbn(Isbn isbn)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Isbn>>, IsbnController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Isbn>>(await negocioIsbn.Value.GuardarIsbnAsync(isbn));
			});
		}
		
		/// <summary>
		/// Metodo editar isbn
		/// </summary>
		/// <param name="isbn">Entidad a editar</param>
		/// <returns>Respuesta tipo Isbn </returns>
		[HttpPut]
		[Route("EditarIsbn")]
		public async Task<Respuesta<Isbn>> EditarIsbn(Isbn isbn)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Isbn>>, IsbnController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Isbn>>(await negocioIsbn.Value.EditarIsbnAsync(isbn));
			});
		}
		
		/// <summary>
		/// Metodo eliminar isbn
		/// </summary>
		/// <param name="isbn">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Isbn </returns>
		[HttpDelete]
		[Route("EliminarIsbn")]
		public async Task<Respuesta<Isbn>> EliminarIsbn(Isbn isbn)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Isbn>>, IsbnController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Isbn>>(await negocioIsbn.Value.EliminarIsbnAsync(isbn));
			});
		}
	}
}

